from fastapi import FastAPI
from langserve import add_routes

class Orchestrator:

    def __init__(self, host: str, port: int, url, **kwargs) -> None:
        self.host = host
        self.port = port
        self.url = url
        self.kwargs = kwargs

    def run(self, debug: bool = False):
        app = FastAPI(
            title="LangChain Server",
            version="1.0",
            description="A simple api server using Langchain's Runnable interfaces",
        )
        for route in self.url.to_json():
            add_routes(app, route["kwargs"]["composer"], path=route["kwargs"]['path'])
        # print(self.url.to_json())
        import uvicorn

        uvicorn.run(app, host=self.host, port=self.port)
        print(f"orchestrator running on {self.host}:{self.port}")


import os, inspect
import xml.etree.ElementTree as ET
from langchain.prompts.chat import (
    ChatPromptTemplate,
    HumanMessagePromptTemplate,
    SystemMessagePromptTemplate,
)

from langchain.prompts import MessagesPlaceholder

class Prompter:
    def __init__(self):
        self.dir = os.path.abspath(os.path.dirname(self.get_caller_path()))

    def get_caller_path(self):
        frame = inspect.stack()[2]
        caller_path = frame.filename
        return caller_path

    def load(self, pathfile, schema, **kwargs):
        tree = ET.parse(os.path.join(self.dir, pathfile))
        root = tree.getroot()
        chat_prompt = []
        for child in root:
            element_name = child.tag
            if element_name == 'system-prompt':
                chat_prompt.append(
                    SystemMessagePromptTemplate.from_template(child.text.strip())
                )
            elif element_name == 'human-prompt':
                chat_prompt.append(
                     HumanMessagePromptTemplate.from_template(child.text.strip())
                )
            elif element_name == "message-placeholder":
                chat_prompt.append(
                    MessagesPlaceholder(variable_name=child.get("variable_name"))
                )
        chat_prompt = ChatPromptTemplate.from_messages(chat_prompt)
        return chat_prompt


from orchestra_llm.core.schema import RouteSchema
import importlib

class Router:
    def __init__(self, **kwargs):
        self.route = []

    def add_route(self, path, composer, schema, name):
        module_name, _, class_name = composer.rpartition('.')
        module = importlib.import_module(module_name)
        composerClass = getattr(module, class_name)

        composer_instance = composerClass()

        composer = composer_instance.run()
        self.route.append(RouteSchema(path=path, composer=composer, input_schema=schema, name=name))

    def to_json(self):
        return [r.to_json() for r in self.route]

def include_urls(file_path, var):
    pass


from langchain_community.vectorstores import Qdrant
from qdrant_client import QdrantClient

class QDrantVectorDB:

    def __init__(self, host, port, collection_name, embeddings):
        self.embeddings = embeddings
        self.collection_name = collection_name
        self.host = host
        self.port = port
        self.client = QdrantClient(self.host, port=self.port)


    def search(self, query:str):
        """
        Searches for relevant documents based on the given query.

        Args:
            query (str): The query string used for searching.

        Returns:
            list: A list of relevant documents.
        """
        vector_store = Qdrant(
            client=self.client,
            collection_name = self.collection_name,
            embeddings = self.embeddings
        )
        retriever = vector_store.as_retriever()
        relevant_documents = retriever.get_relevant_documents(query)
        return relevant_documents

    def get_retriever(self, k=5, **kwargs):
        vector_store = Qdrant(
            client=self.client,
            collection_name = self.collection_name,
            embeddings = self.embeddings
        )
        retriever = vector_store.as_retriever(search_kwargs={"k": k})
        return retriever
    
    def from_documents(self, docs):
        """
        Generate a vector store from a list of documents.

        Parameters:
            docs (list): A list of documents.

        Returns:
            vector_store (Qdrant): The generated vector store.
        """
        vector_store = Qdrant.from_documents(
            docs, 
            self.embeddings, 
            url=self.host, 
            prefer_grpc=True, 
            collection_name = self.collection_name
        )
        return vector_store


# from pydantic import BaseModel
from langchain_core.load.serializable import Serializable
from abc import ABC
from typing import Any, List

# class Schema(BaseModel, ABC):
    

#     def __init__(self, **kwargs: Any) -> None:
#         super().__init__(**kwargs)


#     def to_json(self) -> dict:

#         lc_kwargs = {
#             k: getattr(self, k, v)
#             for k, v in self._lc_kwargs.items()
#             if not (self.__exclude_fields__ or {}).get(k, False)  # type: ignore
#         }

#         return lc_kwargs

class RouteSchema(Serializable):

    path: str
    composer: Any
    input_schema: Any
    name: str

    def __init__(self, **kwargs: Any) -> None:
        """Pass page_content in as positional or named arg."""
        super().__init__(**kwargs)

    @classmethod
    def is_lc_serializable(cls) -> bool:
        """Return whether this class is serializable."""
        return True

    @classmethod
    def get_lc_namespace(cls) -> List[str]:
        """Get the namespace of the langchain object."""
        return ["langchain", "schema", "document"]